#include <stdio.h>
#include <stdlib.h>

#include "matriz.h"
#include "disco.h"

/* ----------- FUNOES ---------------- */

/**********************************************************************
 *  Funcao obtemOrdemMatriz
 * 
 *  Acao:       Obtem de um arquivo a ordem das matrizes que serao 
 *              lidas postriormente pela funcao "obtemMatrizes()"
 *  Entrada:    nenhuma
 *  Saida:      nordem = ordem das matrizes
 *  Retorno:    != 0 se leitura bem sucedida
 *              0 se chegou ao final do arquivo ou erro na ordem da matriz
 *  Suposicoes: No arquivo de entrada existem 2 ou 3 numeros para a ordem.
 *              O primeiro eh linres, o segundo eh collin. O terceiro eh 
 *              opcional e representa colres.
 *              Na ausencia do terceiro numero, assume-se que colres = linres
 **********************************************************************
 */

static int obtemOrdemMatriz (Ordens *nordem)
{
  double *ordem;
  int numdim, i, ok;

  if (ok = numdim = getLinhaDados(&ordem))
  {
    if (numdim < 2 || numdim > 3) {
      ok = NAO;
      fprintf(stderr, "Ordem da matriz invlida:");
      for (i=0; i<numdim; i++)
	fprintf(stderr, " %d", ordem[i]);
      fprintf(stderr, ".\n");
    }
    else {
      nordem->linres = ordem[0];
      nordem->collin = ordem[1];
      
      if (numdim == 2)
	nordem->colres = nordem->linres;
      else
	nordem->colres = ordem[2];
    }
  }

  return (ok);
}


/**********************************************************************
 *  Funcao obtemMatriz
 * 
 *  Acao:       Obtem do usuario matriz linhasXcolunas de um arquivo
 *  Entrada:    linhas e colunas = ordem da matriz
 *  Retorno:    ponteiro para matriz.
 *              Se NULO, erro em alocacao de memoria ou 
 *              numero de colunas incorreto
 *  Algoritmo:  espaco para matriz eh alocado com "calloc()"
 **********************************************************************
 */

static double **obtemMatriz (int linhas, int colunas)
{
  int i;
  double **matriz;
  
  if (matriz = (double **) calloc (linhas, sizeof(double *)))
    for (i = 0; matriz && i < linhas; i++)
      if (getLinhaDados(&matriz[i]) != colunas) {
	fprintf(stderr, "Colunas de matriz nao conferem.\n");	
	matriz = NULL;
      }
  
  return (matriz);
}

/**********************************************************************
 *  Funcao obtemMatrizes
 * 
 *  Acao:       Obtem do usuario matrizes de um arquivo
 *  Entrada:    matrizes = estrutura contendo ordem de matrizes e ponteiros 
 *                         para matrizes que serao lidas do arquivo.
 *  Retorno:    1 se leitura bem sucedida
 *              0 caso contrario
 *  Algoritmo:  espaco para matrizes lidas eh alocado com "calloc()"
 **********************************************************************
 */

int obtemMatrizes (Matrizes *matrizes)
{
  int i, k, ok = NAO;

  if (obtemOrdemMatriz (&matrizes->nordem))
    if (matrizes->mat_01 = obtemMatriz (matrizes->nordem.linres,
					matrizes->nordem.collin))
      if (matrizes->mat_02 = obtemMatriz (matrizes->nordem.collin,
					  matrizes->nordem.colres))
	ok = SIM;
  
  return (ok);
}

/**********************************************************************
 *  Funcao multMatrizes
 * 
 *  Acao:       Efetua multiplicacao entre duas matrizes
 *  Entrada:    mult = estrutura com matrizes a serem multiplicadas,
 *                     suas ordens, e ponteiro para resultado
 *  Retorno:    1 se gerou resultado, 
 *              0 caso contrario
 *  Algoritmo:  espaco para matriz com resultado eh alocada com "calloc()"
 **********************************************************************
 */

int multMatrizes (Matrizes *mult)
{
  int i, j, p, ok = NAO;

  /* Cria e inicia em 0 (zero) a matriz que receber o resultado 
   * da multiplicaao entre mult->mat_01 e mult->mat_02
   */
  if (mult->result = (double **) calloc (mult->nordem.linres, sizeof(double *)))
    for (i=0, ok = SIM; ok && i < mult->nordem.linres; i++)
      if (!(mult->result[i] = (double*) calloc (mult->nordem.colres, sizeof(double))))
	ok = NAO;
    
  /* Efetua a multiplicaao */
  if (ok)
    for (i=0; i < mult->nordem.linres; i++)
      for (j=0; j < mult->nordem.colres; j++)
	for (p=0; p < mult->nordem.collin; p++)
	  mult->result[i][j] += mult->mat_01[i][p] * mult->mat_02[p][j];
  else
    fprintf (stderr, "Erro na multiplicao das matrizes\n");

  return (ok);
}

/**********************************************************************
 *  Funcao imprimeMatriz
 * 
 *  Acao:       Imprime o conteudo de uma matriz em stdout
 *  Entrada:    ponteiro para matriz, ordem da matriz (linhas e colunas)
 *  Saida:      nenhuma
 *  Suposicoes: ponteiro para matriz deve ser nao nulo
 **********************************************************************
 */

void imprimeMatriz (double **matriz, int linhas, int colunas)
{
  int i, k;
  for (i=0; i < linhas; i++) {
    for (k=0; k < colunas; k++)
      printf("%5.2lf\t", matriz[i][k]);
    printf("\n");
  }
}

